@@ -144,6 +144,18 @@ For the Doctor Watsons of this world, as opposed to the Sherlock
     eqnice!(expected, cmd.stdout());
 });
 
+rgtest!(word_period, |dir: Dir, mut cmd: TestCommand| {
+    dir.create("haystack", "...");
+    cmd.arg("-ow").arg(".").arg("haystack");
+
+    let expected = "\
+.
+.
+.
+";
+    eqnice!(expected, cmd.stdout());
+});
+
 rgtest!(line, |dir: Dir, mut cmd: TestCommand| {
     dir.create("sherlock", SHERLOCK);
     cmd.args(&[
@@ -399,7 +411,7 @@ rgtest!(include_zero, |dir: Dir, mut cmd: TestCommand| {
     cmd.args(&["--count", "--include-zero", "nada"]);
     cmd.assert_err();
 
-    let output = cmd.cmd().output().unwrap();
+    let output = cmd.raw_output();
     let stdout = String::from_utf8_lossy(&output.stdout);
     let expected = "sherlock:0\n";
 
@@ -411,7 +423,7 @@ rgtest!(include_zero_override, |dir: Dir, mut cmd: TestCommand| {
     cmd.args(&["--count", "--include-zero", "--no-include-zero", "nada"]);
     cmd.assert_err();
 
-    let output = cmd.cmd().output().unwrap();
+    let output = cmd.raw_output();
     let stdout = String::from_utf8_lossy(&output.stdout);
     assert!(stdout.is_empty());
 });
@@ -788,7 +800,7 @@ rgtest!(unrestricted3, |dir: Dir, mut cmd: TestCommand| {
     cmd.arg("-uuu").arg("foo");
 
     let expected = "\
-Binary file hay matches (found \"\\u{0}\" byte around offset 3)
+hay: binary file matches (found \"\\0\" byte around offset 3)
 ";
     eqnice!(expected, cmd.stdout());
 });
@@ -1001,7 +1013,7 @@ rgtest!(binary_convert, |dir: Dir, mut cmd: TestCommand| {
     cmd.arg("--no-mmap").arg("foo").arg("file");
 
     let expected = "\
-Binary file matches (found \"\\u{0}\" byte around offset 3)
+binary file matches (found \"\\0\" byte around offset 3)
 ";
     eqnice!(expected, cmd.stdout());
 });
@@ -1011,7 +1023,7 @@ rgtest!(binary_convert_mmap, |dir: Dir, mut cmd: TestCommand| {
     cmd.arg("--mmap").arg("foo").arg("file");
 
     let expected = "\
-Binary file matches (found \"\\u{0}\" byte around offset 3)
+binary file matches (found \"\\0\" byte around offset 3)
 ";
     eqnice!(expected, cmd.stdout());
 });
@@ -1065,3 +1077,54 @@ rgtest!(type_list, |_: Dir, mut cmd: TestCommand| {
     // This can change over time, so just make sure we print something.
     assert!(!cmd.stdout().is_empty());
 });
+
+// The following series of tests seeks to test all permutations of ripgrep's
+// sorted queries.
+//
+// They all rely on this setup function, which sets up this particular file
+// structure with a particular creation order:
+//  ├── a             # 1
+//  ├── b             # 4
+//  └── dir           # 2
+//     ├── c          # 3
+//     └── d          # 5
+//
+// This order is important when sorting them by system time-stamps.
+fn sort_setup(dir: Dir) {
+    use std::{thread::sleep, time::Duration};
+
+    let sub_dir = dir.path().join("dir");
+    dir.create("a", "test");
+    sleep(Duration::from_millis(100));
+    dir.create_dir(&sub_dir);
+    sleep(Duration::from_millis(100));
+    dir.create(sub_dir.join("c"), "test");
+    sleep(Duration::from_millis(100));
+    dir.create("b", "test");
+    sleep(Duration::from_millis(100));
+    dir.create(sub_dir.join("d"), "test");
+}
+
+rgtest!(sort_files, |dir: Dir, mut cmd: TestCommand| {
+    sort_setup(dir);
+    let expected = "a:test\nb:test\ndir/c:test\ndir/d:test\n";
+    eqnice!(expected, cmd.args(["--sort", "path", "test"]).stdout());
+});
+
+rgtest!(sort_accessed, |dir: Dir, mut cmd: TestCommand| {
+    if crate::util::is_cross() {
+        return;
+    }
+    sort_setup(dir);
+    let expected = "a:test\ndir/c:test\nb:test\ndir/d:test\n";
+    eqnice!(expected, cmd.args(["--sort", "accessed", "test"]).stdout());
+});
+
+rgtest!(sortr_accessed, |dir: Dir, mut cmd: TestCommand| {
+    if crate::util::is_cross() {
+        return;
+    }
+    sort_setup(dir);
+    let expected = "dir/d:test\nb:test\ndir/c:test\na:test\n";
+    eqnice!(expected, cmd.args(["--sortr", "accessed", "test"]).stdout());
+});
