@@ -380,6 +380,7 @@ rgtest!(r428_color_context_path, |dir: Dir, mut cmd: TestCommand| {
         "-N",
         "--colors=match:none",
         "--color=always",
+        "--hyperlink-format=",
         "foo",
     ]);
 
@@ -392,13 +393,16 @@ rgtest!(r428_color_context_path, |dir: Dir, mut cmd: TestCommand| {
 });
 
 // See: https://github.com/BurntSushi/ripgrep/issues/428
-rgtest!(r428_unrecognized_style, |_: Dir, mut cmd: TestCommand| {
+rgtest!(r428_unrecognized_style, |dir: Dir, mut cmd: TestCommand| {
+    dir.create("file.txt", "Sherlock");
+
     cmd.arg("--colors=match:style:").arg("Sherlock");
     cmd.assert_err();
 
-    let output = cmd.cmd().output().unwrap();
+    let output = cmd.raw_output();
     let stderr = String::from_utf8_lossy(&output.stderr);
     let expected = "\
+rg: error parsing flag --colors: \
 unrecognized style attribute ''. Choose from: nobold, bold, nointense, \
 intense, nounderline, underline.
 ";
@@ -742,6 +746,15 @@ rgtest!(r1259_drop_last_byte_nonl, |dir: Dir, mut cmd: TestCommand| {
     eqnice!("fz\n", cmd.arg("-f").arg("patterns-nl").arg("test").stdout());
 });
 
+// See: https://github.com/BurntSushi/ripgrep/issues/1311
+rgtest!(r1311_multi_line_term_replace, |dir: Dir, mut cmd: TestCommand| {
+    dir.create("input", "hello\nworld\n");
+    eqnice!(
+        "1:hello?world?\n",
+        cmd.args(&["-U", "-r?", "-n", "\n", "input"]).stdout()
+    );
+});
+
 // See: https://github.com/BurntSushi/ripgrep/issues/1319
 rgtest!(r1319, |dir: Dir, mut cmd: TestCommand| {
     dir.create("input", "CCAGCTACTCGGGAGGCTGAGGCTGGAGGATCGCTTGAGTCCAGGAGTTC");
@@ -761,6 +774,28 @@ rgtest!(r1334_crazy_literals, |dir: Dir, mut cmd: TestCommand| {
     );
 });
 
+// See: https://github.com/BurntSushi/ripgrep/issues/1380
+rgtest!(r1380, |dir: Dir, mut cmd: TestCommand| {
+    dir.create(
+        "foo",
+        "\
+a
+b
+c
+d
+e
+d
+e
+d
+e
+d
+e
+",
+    );
+
+    eqnice!("d\ne\nd\n", cmd.args(&["-A2", "-m1", "d", "foo"]).stdout());
+});
+
 // See: https://github.com/BurntSushi/ripgrep/issues/1389
 rgtest!(r1389_bad_symlinks_no_biscuit, |dir: Dir, mut cmd: TestCommand| {
     dir.create_dir("mydir");
@@ -773,6 +808,44 @@ rgtest!(r1389_bad_symlinks_no_biscuit, |dir: Dir, mut cmd: TestCommand| {
     eqnice!("mylink/file.txt:test\n", stdout);
 });
 
+// See: https://github.com/BurntSushi/ripgrep/issues/1401
+rgtest!(r1401_look_ahead_only_matching_1, |dir: Dir, mut cmd: TestCommand| {
+    // Only PCRE2 supports look-around.
+    if !dir.is_pcre2() {
+        return;
+    }
+    dir.create("ip.txt", "foo 42\nxoyz\ncat\tdog\n");
+    cmd.args(&["-No", r".*o(?!.*\s)", "ip.txt"]);
+    eqnice!("xo\ncat\tdo\n", cmd.stdout());
+
+    let mut cmd = dir.command();
+    cmd.args(&["-No", r".*o(?!.*[ \t])", "ip.txt"]);
+    eqnice!("xo\ncat\tdo\n", cmd.stdout());
+});
+
+// See: https://github.com/BurntSushi/ripgrep/issues/1401
+rgtest!(r1401_look_ahead_only_matching_2, |dir: Dir, mut cmd: TestCommand| {
+    // Only PCRE2 supports look-around.
+    if !dir.is_pcre2() {
+        return;
+    }
+    dir.create("ip.txt", "foo 42\nxoyz\ncat\tdog\nfoo");
+    cmd.args(&["-No", r".*o(?!.*\s)", "ip.txt"]);
+    eqnice!("xo\ncat\tdo\nfoo\n", cmd.stdout());
+});
+
+// See: https://github.com/BurntSushi/ripgrep/issues/1412
+rgtest!(r1412_look_behind_no_replacement, |dir: Dir, mut cmd: TestCommand| {
+    // Only PCRE2 supports look-around.
+    if !dir.is_pcre2() {
+        return;
+    }
+
+    dir.create("test", "foo\nbar\n");
+    cmd.args(&["-nU", "-rquux", r"(?<=foo\n)bar", "test"]);
+    eqnice!("2:quux\n", cmd.stdout());
+});
+
 // See: https://github.com/BurntSushi/ripgrep/pull/1446
 rgtest!(
     r1446_respect_excludes_in_worktree,
@@ -864,3 +937,283 @@ use B;
     ]);
     eqnice!("2\n", cmd.stdout());
 });
+
+// See: https://github.com/BurntSushi/ripgrep/issues/1638
+//
+// Tests if UTF-8 BOM is sniffed, then the column index is correct.
+rgtest!(r1638, |dir: Dir, mut cmd: TestCommand| {
+    dir.create_bytes("foo", b"\xef\xbb\xbfx");
+
+    eqnice!("foo:1:1:x\n", cmd.arg("--column").arg("x").stdout());
+});
+
+// See: https://github.com/BurntSushi/ripgrep/issues/1739
+rgtest!(r1739_replacement_lineterm_match, |dir: Dir, mut cmd: TestCommand| {
+    dir.create("test", "a\n");
+    cmd.args(&[r"-r${0}f", r".*", "test"]);
+    eqnice!("af\n", cmd.stdout());
+});
+
+// See: https://github.com/BurntSushi/ripgrep/issues/1757
+rgtest!(f1757, |dir: Dir, _: TestCommand| {
+    dir.create_dir("rust/target");
+    dir.create(".ignore", "rust/target");
+    dir.create("rust/source.rs", "needle");
+    dir.create("rust/target/rustdoc-output.html", "needle");
+
+    let args = &["--files-with-matches", "needle", "rust"];
+    eqnice!("rust/source.rs\n", dir.command().args(args).stdout());
+    let args = &["--files-with-matches", "needle", "./rust"];
+    eqnice!("./rust/source.rs\n", dir.command().args(args).stdout());
+});
+
+// See: https://github.com/BurntSushi/ripgrep/issues/1765
+rgtest!(r1765, |dir: Dir, mut cmd: TestCommand| {
+    dir.create("test", "\n");
+    // We need to add --color=always here to force the failure, since the bad
+    // code path is only triggered when colors are enabled.
+    cmd.args(&[r"x?", "--crlf", "--color", "always"]);
+
+    assert!(!cmd.stdout().is_empty());
+});
+
+// See: https://github.com/BurntSushi/ripgrep/issues/1838
+rgtest!(r1838_nul_error_with_binary_detection, |dir: Dir, _: TestCommand| {
+    // We don't support this error reporting with PCRE2 since we can't parse
+    // the pattern (easily) to give a good error message.
+    if dir.is_pcre2() {
+        return;
+    }
+    dir.create("test", "foo\n");
+
+    dir.command().args(&[r"foo\x00?"]).assert_err();
+    eqnice!("test:foo\n", dir.command().args(&["-a", r"foo\x00?"]).stdout());
+});
+
+// See: https://github.com/BurntSushi/ripgrep/issues/1866
+rgtest!(r1866, |dir: Dir, mut cmd: TestCommand| {
+    dir.create("test", "foobar\nfoobar\nfoo quux");
+    cmd.args(&[
+        "--multiline",
+        "--vimgrep",
+        r"foobar\nfoobar\nfoo|quux",
+        "test",
+    ]);
+
+    // vimgrep only wants the first line of each match, even when a match
+    // spans multiple lines.
+    //
+    // See: https://github.com/BurntSushi/ripgrep/issues/1866
+    let expected = "\
+test:1:1:foobar
+test:3:5:foo quux
+";
+    eqnice!(expected, cmd.stdout());
+});
+
+// See: https://github.com/BurntSushi/ripgrep/issues/1868
+rgtest!(r1868_context_passthru_override, |dir: Dir, _: TestCommand| {
+    dir.create("test", "foo\nbar\nbaz\nquux\n");
+
+    let args = &["-C1", "bar", "test"];
+    eqnice!("foo\nbar\nbaz\n", dir.command().args(args).stdout());
+    let args = &["--passthru", "bar", "test"];
+    eqnice!("foo\nbar\nbaz\nquux\n", dir.command().args(args).stdout());
+
+    let args = &["--passthru", "-C1", "bar", "test"];
+    eqnice!("foo\nbar\nbaz\n", dir.command().args(args).stdout());
+    let args = &["-C1", "--passthru", "bar", "test"];
+    eqnice!("foo\nbar\nbaz\nquux\n", dir.command().args(args).stdout());
+
+    let args = &["--passthru", "-B1", "bar", "test"];
+    eqnice!("foo\nbar\n", dir.command().args(args).stdout());
+    let args = &["-B1", "--passthru", "bar", "test"];
+    eqnice!("foo\nbar\nbaz\nquux\n", dir.command().args(args).stdout());
+
+    let args = &["--passthru", "-A1", "bar", "test"];
+    eqnice!("bar\nbaz\n", dir.command().args(args).stdout());
+    let args = &["-A1", "--passthru", "bar", "test"];
+    eqnice!("foo\nbar\nbaz\nquux\n", dir.command().args(args).stdout());
+});
+
+rgtest!(r1878, |dir: Dir, _: TestCommand| {
+    dir.create("test", "a\nbaz\nabc\n");
+
+    // Since ripgrep enables (?m) by default, '^' will match at the beginning
+    // of a line, even when -U/--multiline is used.
+    let args = &["-U", "--no-mmap", r"^baz", "test"];
+    eqnice!("baz\n", dir.command().args(args).stdout());
+    let args = &["-U", "--mmap", r"^baz", "test"];
+    eqnice!("baz\n", dir.command().args(args).stdout());
+
+    // But when (?-m) is disabled, or when \A is used, then there should be no
+    // matches that aren't anchored to the beginning of the file.
+    let args = &["-U", "--no-mmap", r"(?-m)^baz", "test"];
+    dir.command().args(args).assert_err();
+    let args = &["-U", "--mmap", r"(?-m)^baz", "test"];
+    dir.command().args(args).assert_err();
+
+    let args = &["-U", "--no-mmap", r"\Abaz", "test"];
+    dir.command().args(args).assert_err();
+    let args = &["-U", "--mmap", r"\Abaz", "test"];
+    dir.command().args(args).assert_err();
+});
+
+// See: https://github.com/BurntSushi/ripgrep/issues/1891
+rgtest!(r1891, |dir: Dir, mut cmd: TestCommand| {
+    dir.create("test", "\n##\n");
+    // N.B. We use -o here to force the issue to occur, which seems to only
+    // happen when each match needs to be detected.
+    eqnice!("1:\n2:\n2:\n2:\n", cmd.args(&["-won", "", "test"]).stdout());
+});
+
+// See: https://github.com/BurntSushi/ripgrep/issues/2095
+rgtest!(r2095, |dir: Dir, mut cmd: TestCommand| {
+    dir.create(
+        "test",
+        "#!/usr/bin/env bash
+
+zero=one
+
+a=one
+
+if true; then
+	a=(
+		a
+		b
+		c
+	)
+	true
+fi
+
+a=two
+
+b=one
+});
+",
+    );
+    cmd.args(&[
+        "--line-number",
+        "--multiline",
+        "--only-matching",
+        "--replace",
+        "${value}",
+        r"^(?P<indent>\s*)a=(?P<value>(?ms:[(].*?[)])|.*?)$",
+        "test",
+    ]);
+    let expected = "4:one
+8:(
+9:		a
+10:		b
+11:		c
+12:	)
+15:two
+";
+    eqnice!(expected, cmd.stdout());
+});
+
+// See: https://github.com/BurntSushi/ripgrep/issues/2198
+rgtest!(r2198, |dir: Dir, mut cmd: TestCommand| {
+    dir.create(".ignore", "a");
+    dir.create(".rgignore", "b");
+    dir.create("a", "");
+    dir.create("b", "");
+    dir.create("c", "");
+
+    cmd.arg("--files").arg("--sort").arg("path");
+    eqnice!("c\n", cmd.stdout());
+    eqnice!("a\nb\nc\n", cmd.arg("--no-ignore-dot").stdout());
+});
+
+// See: https://github.com/BurntSushi/ripgrep/issues/2208
+rgtest!(r2208, |dir: Dir, mut cmd: TestCommand| {
+    dir.create("test", "# Compile requirements.txt files from all found or specified requirements.in files (compile).
+# Use -h to include hashes, -u dep1,dep2... to upgrade specific dependencies, and -U to upgrade all.
+pipc () {  # [-h] [-U|-u <pkgspec>[,<pkgspec>...]] [<reqs-in>...] [-- <pip-compile-arg>...]
+    emulate -L zsh
+    unset REPLY
+    if [[ $1 == --help ]] { zpy $0; return }
+    [[ $ZPY_PROCS ]] || return
+
+    local gen_hashes upgrade upgrade_csv
+    while [[ $1 == -[hUu] ]] {
+        if [[ $1 == -h ]] { gen_hashes=--generate-hashes; shift   }
+        if [[ $1 == -U ]] { upgrade=1;                    shift   }
+        if [[ $1 == -u ]] { upgrade=1; upgrade_csv=$2;    shift 2 }
+    }
+}
+");
+    cmd.args(&[
+        "-N",
+        "-U",
+        "-r", "$usage",
+        r#"^(?P<predoc>\n?(# .*\n)*)(alias (?P<aname>pipc)="[^"]+"|(?P<fname>pipc) \(\) \{)(  #(?P<usage> .+))?"#,
+        "test",
+    ]);
+    let expected = " [-h] [-U|-u <pkgspec>[,<pkgspec>...]] [<reqs-in>...] [-- <pip-compile-arg>...]\n";
+    eqnice!(expected, cmd.stdout());
+});
+
+// See: https://github.com/BurntSushi/ripgrep/issues/2236
+rgtest!(r2236, |dir: Dir, mut cmd: TestCommand| {
+    dir.create(".ignore", r"foo\/");
+    dir.create_dir("foo");
+    dir.create("foo/bar", "test\n");
+    cmd.args(&["test"]).assert_err();
+});
+
+// See: https://github.com/BurntSushi/ripgrep/issues/2480
+rgtest!(r2480, |dir: Dir, mut cmd: TestCommand| {
+    dir.create("file", "FooBar\n");
+
+    // no regression in empty pattern behavior
+    cmd.args(&["-e", "", "file"]);
+    eqnice!("FooBar\n", cmd.stdout());
+
+    // no regression in single pattern behavior
+    let mut cmd = dir.command();
+    cmd.args(&["-e", ")(", "file"]);
+    eqnice!("FooBar\n", cmd.stdout());
+
+    // no regression in multiple patterns behavior
+    let mut cmd = dir.command();
+    cmd.args(&["--only-matching", "-e", "Foo", "-e", "Bar", "file"]);
+    eqnice!("Foo\nBar\n", cmd.stdout());
+
+    // no regression in capture groups behavior
+    let mut cmd = dir.command();
+    cmd.args(&["-e", "Fo(oB)a(r)", "--replace", "${0}_${1}_${2}${3}", "file"]);
+    eqnice!("FooBar_oB_r\n", cmd.stdout()); // note: ${3} expected to be empty
+
+    // flag does not leak into next pattern on match
+    let mut cmd = dir.command();
+    cmd.args(&["--only-matching", "-e", "(?i)foo", "-e", "bar", "file"]);
+    eqnice!("Foo\n", cmd.stdout());
+
+    // flag does not leak into next pattern on mismatch
+    let mut cmd = dir.command();
+    cmd.args(&["--only-matching", "-e", "(?i)notfoo", "-e", "bar", "file"]);
+    cmd.assert_err();
+});
+
+// See: https://github.com/BurntSushi/ripgrep/issues/2574
+rgtest!(r2574, |dir: Dir, mut cmd: TestCommand| {
+    dir.create("haystack", "some.domain.com\nsome.domain.com/x\n");
+    let got = cmd
+        .args(&[
+            "--no-filename",
+            "--no-unicode",
+            "-w",
+            "-o",
+            r"(\w+\.)*domain\.(\w+)",
+        ])
+        .stdout();
+    eqnice!("some.domain.com\nsome.domain.com\n", got);
+});
+
+// See: https://github.com/BurntSushi/ripgrep/issues/2658
+rgtest!(r2658_null_data_line_regexp, |dir: Dir, mut cmd: TestCommand| {
+    dir.create("haystack", "foo\0bar\0quux\0");
+    let got = cmd.args(&["--null-data", "--line-regexp", r"bar"]).stdout();
+    eqnice!("haystack:bar\0", got);
+});
